% This code is part of the article:
% How to tackle underdeterminacy in metabolic flux analysis? A tutorial and critical review
% copyright Ph. Bogaerts, A.Vande Wouwer
% Processes, 2021

function [q]=hit_and_run(A,b,q0,nSamples_HR)

size_q=size(A,2);
q=zeros(size_q,nSamples_HR);

h = waitbar(0,'Computing hit-and-run samples ...');
for i=1:nSamples_HR,

    % Determination of the u direction for the RDA algorithm
    u=rand(size_q,1);
    u=u/norm(u);
    
    % Determination of the u direction for the CDA algorithm 
%     ui=round(1+(size_q-1)*rand);
%     u=zeros(size_q,1);
%     u(ui,1)=1;
    
    lambda=zeros(size(A,1),1);
    for j=1:size(A,1),
        lambda(j,1)=(b(j)-A(j,:)*q0)/(A(j,:)*u);
    end
    lambda(isinf(lambda))=[];
    lambda_min=max(lambda(lambda<=0));
    lambda_max=min(lambda(lambda>=0));
    lambda_new=lambda_min+rand*(lambda_max-lambda_min);
    
    q0=q0+lambda_new*u;
    
    q(:,i)=q0;
   
    waitbar(i/nSamples_HR);
end
close(h)
