% This code is part of the article:
% How to tackle underdeterminacy in metabolic flux analysis? A tutorial and critical review
% copyright Ph. Bogaerts, A.Vande Wouwer
% Processes, 2021

function [q, Q_min, Q_max, test] = rejection_algorithm(A,b,nSamples)

size_q = size(A,2);

lb = -1e6*ones(size_q,1);
ub = 1e6*ones(size_q,1);

Q_min=zeros(size_q,1);
Q_max=zeros(size_q,1);
for i=1:size_q,
    f=zeros(1,size_q);
    f(i)=1;
    [x,fval]=cplexlp(f,A,b,[],[],lb,ub);
    Q_min(i,1)=fval;

    f=zeros(1,size_q);
    f(i)=-1;
    [x,fval]=cplexlp(f,A,b,[],[],lb,ub);
    Q_max(i,1)=-fval;
end

q=zeros(nSamples,size_q);
m=0;
test=0;

h = waitbar(0,'Computing rejection algorithm samples ...');
while m < nSamples,
    m=m+1;
    reject=1;
    while reject==1,
        test=test+1;
        q_test=Q_min+rand(size_q,1).*(Q_max-Q_min);
        % tol=1e-7;
        tol=0;
        if max(A*q_test-b)<=tol,
            reject=0;
        end
    end
    q(m,:)=q_test';
    waitbar(m/nSamples);
end
close(h)

q=q';
